#include <SoftwareSerial.h>
#include <DHT.h>

#define DHTPIN 2       // Pinul la care este conectat senzorul DHT11
#define DHTTYPE DHT11  // Tipul senzorului DHT11
#define RX_PIN 10
#define TX_PIN 9  // Pinii folosiți pentru comunicația SoftwareSerial
#define LED_PIN 13
#define lockPin 12
#define unlockPin 11
#define buzzerPin 6  // Pinul pentru buzzer
#define TEMP_THRESHOLD 30  // Pragul de temperatură pentru deblocare și activarea buzzerului

SoftwareSerial mySerial(RX_PIN, TX_PIN);
DHT dht(DHTPIN, DHTTYPE);

void setup() {
  pinMode(LED_PIN, OUTPUT);    // Pinul LED setat ca ieșire pentru testarea primirii datelor
  pinMode(lockPin, OUTPUT);    // Pinul pentru blocare setat ca ieșire
  pinMode(unlockPin, OUTPUT);  // Pinul pentru deblocare setat ca ieșire
  pinMode(buzzerPin, OUTPUT);  // Pinul pentru buzzer setat ca ieșire
  
  // Ensure the initial states are LOW to prevent the buzzer from ringing at startup
  digitalWrite(LED_PIN, LOW);
  digitalWrite(lockPin, LOW);
  digitalWrite(unlockPin, LOW);
  noTone(buzzerPin);  // Ensure buzzer is off at the start

  Serial.begin(9600);          // Inițializează comunicația serială încorporată
  mySerial.begin(9600);        // Inițializează comunicația SoftwareSerial

  dht.begin();                 // Inițializează senzorul DHT
  Serial.println("Setup complete. Awaiting commands...");
  mySerial.println("Setup complete. Awaiting commands...");
}

void loop() {
  if (mySerial.available()) {
    digitalWrite(LED_PIN, HIGH); // Pornește LED-ul dacă primește date
    String command = mySerial.readStringUntil('\n');
    command.trim();
    Serial.println("Received command: " + command);
    mySerial.println("Echo: " + command);

    if (command.equalsIgnoreCase("lock")) {
      Serial.println("The door is now locked.");
      mySerial.println("The door is now locked.");
      digitalWrite(lockPin, HIGH);
      delay(3000);
      digitalWrite(lockPin, LOW);
    } else if (command.equalsIgnoreCase("unlock")) {
      Serial.println("The door is now unlocked.");
      mySerial.println("The door is now unlocked.");
      digitalWrite(unlockPin, HIGH);
      delay(3000);
      digitalWrite(unlockPin, LOW);
    } else {
      Serial.println("Unknown command: " + command);
      mySerial.println("Unknown command: " + command);
      tone(buzzerPin, 400); // Activează buzzerul pentru comandă greșită
      delay(1000); // Buzzer sună timp de 1 secundă
      noTone(buzzerPin);  // Dezactivează buzzerul
    }
    digitalWrite(LED_PIN, LOW); // Oprește LED-ul după procesare
  }

  // Read temperature from the DHT11 sensor
  float temperatureC = dht.readTemperature();

  // Check if the reading is valid
  if (isnan(temperatureC)) {
    Serial.println("Failed to read from DHT sensor!");
    return;
  }
  
  // Print temperature to Serial Monitor
  Serial.print("Temperature: ");
  Serial.print(temperatureC);
  Serial.println(" C");

  // Check if temperature exceeds the threshold
  if (temperatureC > TEMP_THRESHOLD) {
    Serial.println("Temperature threshold exceeded! Unlocking door and activating buzzer.");
    digitalWrite(unlockPin, HIGH); // Deblochează ușa
    tone(buzzerPin, 400); // Activează buzzerul
    delay(3000); // Așteaptă 3 secunde
    digitalWrite(unlockPin, LOW);  // Oprește deblocarea
    noTone(buzzerPin);  // Dezactivează buzzerul
  }

  delay(1000); // Wait for 1 second before next reading

  if (Serial.available()) {
    char c = Serial.read();
    mySerial.write(c);
  }
}
